package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.yonge.cooleshow.biz.dal.entity.LessonCourseware;
import com.yonge.cooleshow.biz.dal.service.LessonCoursewareService;
import com.yonge.cooleshow.biz.dal.wrapper.LessonCoursewareWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@RestController
@RequestMapping("${app-config.url.admin:}/lessonCourseware")
@Api(value = "课件表", tags = "课件表")
public class LessonCoursewareController extends BaseController {

    @Resource
    private LessonCoursewareService lessonCoursewareService;
    @Resource
    private CoursewareFeignService coursewareFeignService;

	@ApiOperation(value = "获取内容平台课件教材")
	@PostMapping("/queryPage")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/queryPage')")
	public HttpResponseResult<PageInfo<CbsLessonCoursewareWrapper.LessonCourseware>> queryPage(@RequestBody CbsLessonCoursewareWrapper.LessonCoursewareQuery query) throws Exception {
		query.setEnableFlag(true);
		return succeed(pageInfo(coursewareFeignService.lessonCoursewarePage(query).feignData()));
	}

	public static <T> PageInfo<T> pageInfo(com.microsvc.toolkit.common.response.paging.PageInfo<T> source) {
		if (Objects.isNull(source)) {
			return new PageInfo<>(1, 20);
		}
		int total = Integer.parseInt(String.valueOf(source.getTotal()));
		int limit = Integer.parseInt(String.valueOf(source.getLimit()));
		PageInfo<T> resultPage = new PageInfo<>();
		resultPage.setRows(source.getRows());
		resultPage.setPageNo(Long.valueOf(source.getCurrent()).intValue());
		resultPage.setLimit(limit);
		resultPage.setTotal(total);
		return resultPage;
	}

	@ApiOperation(value = "分页查询已添加的课件")
	@PostMapping("/queryLessonCourseware")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/queryLessonCourseware')")
	public HttpResponseResult<PageInfo<LessonCoursewareWrapper.LessonCoursewareDto>> queryLessonCourseware(@RequestBody LessonCoursewareWrapper.LessonCoursewareQuery query){
		IPage<LessonCoursewareWrapper.LessonCoursewareDto> pages = lessonCoursewareService.selectPage(QueryInfo.getPage(query), query);
		return succeed(PageUtil.pageInfo(pages));
	}

	@ApiOperation(value = "获取已添加的课件id列表")
	@PostMapping("/queryLessonCoursewareId")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/queryLessonCoursewareId')")
	public HttpResponseResult<List<Long>> queryLessonCoursewareId(){
		List<LessonCourseware> list = lessonCoursewareService.lambdaQuery().list();
		if (list == null || list.isEmpty()) {
			return succeed(Lists.newArrayList());
		}
		List<Long> ids = list.stream().map(LessonCourseware::getLessonCourseId).collect(java.util.stream.Collectors.toList());
		return succeed(ids);
	}

	@ApiOperation(value = "添加课件")
	@PostMapping("/addLessonCourseware")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/addLessonCourseware')")
	public HttpResponseResult<Boolean> addLessonCourseware(@RequestBody List<Long> lessonCoursewareIds){
		if (CollectionUtils.isEmpty(lessonCoursewareIds)) {
			throw new BizException("课件id不能为空");
		}
		List<LessonCourseware> list = new ArrayList<>();
		lessonCoursewareIds.forEach(id -> {
			LessonCourseware lessonCourseware = new LessonCourseware();
			lessonCourseware.setLessonCourseId(id);
			list.add(lessonCourseware);
		});
		Integer count = lessonCoursewareService.lambdaQuery().in(LessonCourseware::getLessonCourseId, lessonCoursewareIds).count();
		if (count > 0) {
			throw new BizException("请勿重复添加课件");
		}
		return succeed(lessonCoursewareService.saveBatch(list));
	}

	@ApiOperation(value = "批量删除已添加的课件")
	@PostMapping("/deleteLessonCourseware")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/deleteLessonCourseware')")
	public HttpResponseResult<Boolean> deleteLessonCourseware(@RequestBody List<Long> ids){
		if (CollectionUtils.isEmpty(ids)) {
			return succeed(false);
		}
		return succeed(lessonCoursewareService.removeByIds(ids));
	}

	@ApiOperation(value = "启用、停用课件")
	@GetMapping("/updateEnableFlag")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/updateEnableFlag')")
	public HttpResponseResult updateEnableFlag(Integer id,Boolean enableFlag){
		lessonCoursewareService.lambdaUpdate().set(LessonCourseware::getEnable, enableFlag)
				.eq(LessonCourseware::getId, id)
				.update();
		return succeed();
	}

	@ApiOperation(value = "修改课件声部")
	@GetMapping("/updateSubject")
	@PreAuthorize("@pcs.hasPermissions('lessonCourseware/updateSubject')")
	public HttpResponseResult updateSubject(Integer id,Integer subjectId){
		lessonCoursewareService.lambdaUpdate().set(LessonCourseware::getSubjectId, subjectId)
				.eq(LessonCourseware::getId, id)
				.update();
		return succeed();
	}
}